:li
/* rech1.c
 * ===============================================================
 * Simple Calculator with Variables (95/05/02)
 * Martin Weitzel, Darmstadt
 * ===============================================================
*/

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "icheck.h"

void showhelp(void)	/* -- explain commands */
{
	(void) printf(
		"Valid input syntax:\n"
		"(v is a variable [a-z], cv a value or a\n"
		"variable, and op an arithmetic operation\n" 
		"?         -- show this help\n"
		"hex       -- output hexadecimal\n"
		"dec       -- output decimal\n"
		"oct       -- output octal\n"
		"cv        -- show value\n"
		"cv op cv  -- show result of operation\n"
		"v = ...   -- assign result of ...\n"
		"quit      -- end this program\n"
	);
}

#define VAR	"%1[abcdefghijklmnopqrstuvwxyz]"
#define CONST	"%i"
#define _op_	" %1[-+/*%] "

static int vars[26];	/* -- 26 user variables */
#define v(x) (vars[*(x)-'a'])	/* access macro */

int calc(		/* -- arithmetic operation */
	char op,		/* operation */
	int v1,			/* 1st operand */
	int v2			/* 2nd operand */
) {
	switch (op) {
	case '+': return v1 + v2;
	case '-': return v1 - v2;
	case '*': return v1 * v2;
	case '/': return v1 / v2;
	case '%': return v1 % v2;
	}
	assert(0); /*NOTREACHED*/
}

int process(		/* -- process single command */
	const char *p		/* command string */
) {
	/* possible output formats */
	static const char HEXFMT[] = "= %#x";
	static const char DECFMT[] = "= %d";
	static const char OCTFMT[] = "= %#o";
	static const char *ofmt = HEXFMT;

	int v1, v2;			/* numerical input */
	char op[2];			/* operation */
	char n[2], n1[2], n2[2];	/* variable names */

	assert(p != (const char *)0);

	p += strspn(p, " \t");	/* skip leading space */

	if CHK0(p, "") return 0;/* empty line */

	/*
	 * select output format
	*/
	if CHK0(p, "hex") { ofmt = HEXFMT; return 0; }
	if CHK0(p, "dec") { ofmt = DECFMT; return 0; }
	if CHK0(p, "oct") { ofmt = OCTFMT; return 0; }

	/*
	 * show value, variable or result of simple
	 * expression according to selected format
	*/
	if CHK1(p, CONST, &v1) {
		(void) printf(ofmt, v1);
		return 1;
	}
	if CHK3(p, CONST _op_ CONST, &v1, op, &v2) {
		(void) printf(ofmt, calc(*op, v1, v2));
		return 1;
	}
	if CHK1(p, VAR , n1) {
		(void) printf(ofmt, v(n1));
		return 1;
	}
	if CHK3(p, VAR _op_ CONST, n1, op, &v2) {
		(void) printf(ofmt, calc(*op, v(n1), v2));
		return 1;
	}
	if CHK3(p, VAR _op_ VAR , n1, op, n2) {
		(void) printf(ofmt, calc(*op, v(n1), v(n2)));
		return 1;
	}

	/*
	 * assign to variable (no output)
	*/
	if CHK2(p, VAR " = " CONST, n, &v1) {
		v(n) = v1;
		return 0;
	}
	if CHK2(p, VAR " = " VAR, n, n1) {
		v(n) = v(n1);
		return 0;
	}
	if CHK4(p, VAR " = " CONST _op_ CONST,
			n, &v1, op, &v2) {
		v(n) = calc(*op, v1, v2);
		return 0;
	}
	if CHK4(p, VAR " = " VAR _op_ CONST,
			n, n1, op, &v2) {
		v(n) = calc(*op, v(n1), v2);
		return 0;
	}
	if CHK4(p, VAR " = " VAR _op_ VAR,
			n, n1, op, n2) {
		v(n) = calc(*op, v(n1), v(n2));
		return 0;
	}

	/*
	 * terminate with with "quit" (also some common
	 * abbreviations are accepted for convenience)
	*/
	if (CHK0(p, "quit")
	 || CHK0(p, "%*1[qex]%*s")) return EOF;

	/*
	 * give some hints about input syntax
	*/
	if (CHK0(p, "?")
	 || CHK0(p, "help")) { showhelp(); return 0; }

	/*
	 * everything else is a ...
	*/
	(void) printf(": syntax error");
	return 2;
}

static char line[80];	/* -- buffer for single command */

void interactive(void)	/* -- interactive operation mode */
{
	while (fgets(line, sizeof line, stdin)) {
		switch (process(line)) {
		case 2:
			(void) printf(" (? lists commands)");
			/*FALLTHROUGH*/
		case 1:
			(void) printf("\n");
			/*FALLTHROUGH*/
		case 0:
			break;
		case EOF:
			return;
		}
	}
}

void commandline(	/* -- operate on command line args */
	char **av		/* access args */
) {
	while (*++av) {
		(void) strncpy(line, *av, sizeof line - 2);
		(void) printf("%s ", line);
		(void) process(strcat(line, "\n"));
		(void) printf("\n");
	}
}

int main(		/* -- main program routine */
	int argc,		/* arg count */
	char *argv[]		/* arg vector */
) {
	if (argc == 1)
		interactive();
	else
		commandline(argv);
	return 0;
}
